/*
 * Very simple IR-Controller, which listens on serial port for defined commands and then sends corresponding IR-Commands.
 * Make sure SEND_PIN is defined correctly!
 * The IR-Codes are hard-coded. A better approach would be to add a command to set the desired codes.
 * If you want to keep them hard-coded, make sure to use the right protocol and codes.
 */


#include <Arduino.h>
#include <IRremote.hpp> 
const int SEND_PIN = 2;


decode_results results;



void setup(){
  Serial.begin(9600);
  Serial.setTimeout(50);
  //IrReceiver.begin(RECV_PIN, true);

  pinMode(SEND_PIN, OUTPUT);
  IrSender.begin(SEND_PIN,false, 0);
  IrSender.sendNECRepeat();
  Serial.println("loaded");
}


void loop(){

  /*if (IrReceiver.decode()) {
    IrReceiver.printIRResultShort(&Serial);
    Serial.print("Address: " ); Serial.println(IrReceiver.decodedIRData.address);
    Serial.println();
    IrReceiver.resume();
  }*/


 /*
  * We have to different approaches here..
  * The first approach just sends the command isolated for each time, the char "+" is received.
  * The second approach uses the "Repeat"-Function if the IR_REMOTE lib instead of a loop which a different char.
  */


 if (Serial.available() > 0) {
    // read the incoming byte:
    String incoming = Serial.readString();  //read until timeout
    incoming.trim();
    Serial.print("Received: "); Serial.println(incoming);


     /*
      * Using the loop
      */
    if (incoming.startsWith("+"))
    {
      int le = incoming.length();
      Serial.print("Sending higher "); Serial.print(le); Serial.println(" times.");
      for (int i=0; i < le; i++){
        IrSender.sendNEC(0x3412, 0x22, 1);
        delay(100);
      }
    }
    if (incoming.startsWith("-"))
    {
      int le = incoming.length();
      Serial.print("Sending lower "); Serial.print(le); Serial.println(" times.");
      for (int i=0; i < le; i++){
        IrSender.sendNEC(0x3412, 0x21, 1);
        delay(100);
      }
    }

    /*        
     * Using the Repeat-Function
     */
    if (incoming.startsWith("p"))
    {
      int le = incoming.length();
      Serial.print("Sending higher "); Serial.print(le); Serial.println(" times.");
      IrSender.sendNEC(0x3412, 0x22, le);
      delay(200);
    }
    if (incoming.startsWith("m"))
    {
      int le = incoming.length();
      Serial.print("Sending lower "); Serial.print(le); Serial.println(" times.");
      IrSender.sendNEC(0x3412, 0x21, le);
      delay(200);
    }

  }
}
