'use strict';

var libQ = require('kew');
var fs=require('fs-extra');
var config = new (require('v-conf'))();
var exec = require('child_process').exec;
var execSync = require('child_process').execSync;
var io = require('socket.io-client');
var threshold = 10;
var fallback = 1;
var checkrate = 2;
var socket;

var isRunning = false;

const { SerialPort } = require('serialport')
var sPort;
var incChar;
var decChar;
var port;


module.exports = volumeGuard;
function volumeGuard(context) {
	var self = this;

	this.context = context;
	this.commandRouter = this.context.coreCommand;
	this.logger = this.context.logger;
	this.configManager = this.context.configManager;

}



volumeGuard.prototype.onVolumioStart = function()
{
	var self = this;
	var configFile=this.commandRouter.pluginManager.getConfigurationFile(this.context,'config.json');
	this.config = new (require('v-conf'))();
	this.config.loadFile(configFile);

    return libQ.resolve();
}

volumeGuard.prototype.handleStatus = function (data) {
	var self = this;
	self.logger.info("VolumeGuard::handleStatus - Current Volume: " + data.volume);
}

volumeGuard.prototype.onStart = function() {
    var self = this;
	var defer=libQ.defer();
	self.logger.info("VolumeGuard: Starting Volume Guard..");
	self.socket = io.connect('http://localhost:3000');
	
	/*
	 Get Vars from config
	*/
	self.incChar = this.config.get('increase');
    self.decChar = this.config.get('decrease');
    self.port = this.config.get('port');

	/*
	 Output Vars to log
	*/
	self.logger.info("VolumeGuard::onStart Port: " + self.port);
	self.logger.info("VolumeGuard::onStart IncChar: " + self.incChar);
	self.logger.info("VolumeGuard::onStart DecChar: " + self.decChar);

	self.isRunning = true;

	/*
	Register Callback, each Volume change will trigger this callback
	*/
	self.socket.on('pushState',function(data) {
		self.handleStatusIR(data);
	})

	/*
	Init Serial Port for communication with arduino
	*/
	self.sPort = new SerialPort({
		path: self.port,
		baudRate: 9600,
	  })
	self.sPort.on('error', function(err) {
		self.logger.error('IR Sender::onStart: SerialPort Error -> ', err.message)
	  })

	self.logger.info("IR Sender::onStart - Setting Volume to 50 at StartUp");
	self.socket.emit('volume',50);

	// Once the Plugin has successfull started resolve the promise
	defer.resolve();
	self.logger.info("VolumeGuard: Started Volume Guard");
    return defer.promise;
};

volumeGuard.prototype.onStop = function() {
    var self = this;
    var defer=libQ.defer();
	self.logger.info("VolumeGuard: Stopping Volume Guard..");
    // Once the Plugin has successfull stopped resolve the promise
	self.isRunning = false;
	self.socket.disconnect();
    defer.resolve();
	self.logger.info("VolumeGuard: Stopped Volume Guard");
    return libQ.resolve();
};

volumeGuard.prototype.onRestart = function() {
    var self = this;
    // Optional, use if you need it
};


// Configuration Methods -----------------------------------------------------------------------------

volumeGuard.prototype.getUIConfig = function() {
    var defer = libQ.defer();
    var self = this;

    var lang_code = this.commandRouter.sharedVars.get('language_code');
	self.logger.info('VolumeGuard::getUIConfig - Begin');
    self.commandRouter.i18nJson(__dirname+'/i18n/strings_'+lang_code+'.json',
        __dirname+'/i18n/strings_en.json',
        __dirname + '/UIConfig.json')
        .then(function(uiconf)
        {
			self.logger.info('VolumeGuard::getUIConfig - Loading..');
			uiconf.sections[0].content[0].value = self.config.get('port'); //Port for the arduino is connected to, e.g. /dev/ttyUSB0
            uiconf.sections[0].content[1].value = self.config.get('increase'); //Char which is sent to increase volume
			uiconf.sections[0].content[2].value = self.config.get('decrease'); //Char which is sent to decrease volume
			self.logger.info('VolumeGuard::getUIConfig - Loaded');
			self.logger.info("IR Sender: Loaded Port: " + self.config.get('port'));
			self.logger.info("IR Sender: Loaded Increase: " + self.config.get('increase'));
			self.logger.info("IR Sender: Loaded Decrease: " + self.config.get('decrease'));
			self.logger.info('VolumeGuard::getUIConfig - Success');
            defer.resolve(uiconf);
        })
        .fail(function()
        {
			self.logger.error('VolumeGuard - Failed to parse UI Configuration page: ' + error);
            defer.reject(new Error());
        });

    return defer.promise;
};



volumeGuard.prototype.getConfigurationFiles = function() {
	return ['config.json'];
}

volumeGuard.prototype.setUIConfig = function(data) {
	var self = this;
	//Perform your installation tasks here
};

volumeGuard.prototype.getConf = function(varName) {
	var self = this;
	//Perform your installation tasks here
};

volumeGuard.prototype.setConf = function(varName, varValue) {
	var self = this;
	//Perform your installation tasks here
};


volumeGuard.prototype.saveOptions = function (data) {
    const self = this;

    self.logger.info('VolumeGuard: saving settings');

    self.port = data['port'];
	self.incChar = data['increase'];
	self.decChar = data['decrease'];

	self.logger.info('VolumeGuard: Port ' + self.port);
	self.logger.info('VolumeGuard: IncChar ' + self.incChar);
	self.logger.info('VolumeGuard: DecChar ' + self.decChar);

    self.config.set('port', self.port);
    self.config.set('increase', self.incChar);
	self.config.set('decrease', self.decChar);

    self.commandRouter.pushToastMessage('success', 'VoluemGuard', "Settings saved!");

    self.logger.info('VolumeGuard: settings saved');

    return libQ.resolve();
};

volumeGuard.prototype.handleStatusIR = function (data) {
	var self = this;
	/*
	Debug messages.. Should be removed in production use
	*/
	self.logger.info("IR Sender::handleStatus - Current Volume: " + data.volume);
	self.logger.info("IR Sender::handleStatusIR - incChar: " + self.incChar);
	self.logger.info("IR Sender::handleStatusIR - decChar: " + self.decChar);
	self.logger.info("IR Sender::handleStatusIR - Port: " + self.port);

	/*
	ATTENTION!
	As this plugin is not aware of the current volume, which is set on amp or dac, we assume a volumio standard volume level of 50.
	After each change of volume, the level is reverted back to 50. The positive or negative difference to 50 is change, which will be
	transmitted via IR.
	*/
	if(self.isRunning) {
		if(isNaN(data.volume)) {
			self.logger.info("IR Sender::Loop - Current Volume NaN");
			self.socket.emit('volume',50);
		}
		else {
			if(data.volume > 50) {
				self.logger.info("IR Sender::handleStatus - Volume higher than 50");
				var voldiff = data.volume - 50;
				var message = "";
				for(let i = 0; i<= voldiff; i++) {
					message += self.incChar;
				}
				self.logger.info("IR Sender::handleStatus - Sending: " + message);
				self.sPort.write(message);
				self.socket.emit('volume',50);
			}
			if(data.volume < 50) {
				self.logger.info("IR Sender::handleStatus - Volume lower than 50");
				var voldiff = 50 - data.volume;
				var message = "";
				for(let i = 0; i<= voldiff; i++) {
					message += self.decChar;
				}
				self.logger.info("IR Sender::handleStatus - Sending: " + message);
				self.sPort.write(message);
				self.socket.emit('volume',50);
			}
			
		}
	} 
}



